/*
 * Copyright (c) 2011 by Intevation GmbH, Germany <info@intevation.de>
 *
 * This file is part of MXD2map.
 *
 * This program is free software under the LGPL (>=v2.1)
 * Read the file LICENCE.txt coming with the software for details
 * or visit http://www.gnu.org/licenses/ if it does not exist.
 *
 * MXD2map has been developed on behalf of the
 * Bundesamt fuer Seeschifffahrt und Hydrographie (BSH) in Hamburg
 * by Intevation GmbH.
 *
 * Authors:
 * Raimund Renkert <raimund.renkert@intevation.de>
 * Bjoern Schilberg <bjoern.schilberg@intevation.de>
 * Stephan Holl <stephan.holl@intevation.de>
 */

package de.intevation.mxd.reader;

import org.apache.log4j.Logger;

import com.esri.arcgis.carto.ILayer;
import com.esri.arcgis.carto.RasterLayer;
import com.esri.arcgis.carto.AnnotateLayerPropertiesCollection;
import com.esri.arcgis.carto.IAnnotateLayerProperties;
import com.esri.arcgis.carto.LabelEngineLayerProperties;
import com.esri.arcgis.system.IName;
import com.esri.arcgis.system.IPropertySet;
import com.esri.arcgis.geometry.Envelope;
import com.esri.arcgis.geometry.ISpatialReference;
import com.esri.arcgis.geometry.ProjectedCoordinateSystem;
import com.esri.arcgis.geometry.GeographicCoordinateSystem;
import com.esri.arcgis.geometry.UnknownCoordinateSystem;
import com.esri.arcgis.geometry.Projection;

import org.w3c.dom.Element;

import de.intevation.mxd.utils.MapToXMLUtils;
import java.io.IOException;
import com.esri.arcgis.interop.AutomationException;
/**
 * Reads Layer information.
 *
 * @author <a href="mailto:aheinecke@intevation.de">Andre Heinecke</a>
 */
public class RasterLayerReader
implements ILayerReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(RasterLayerReader.class);

    /**
     * Privte member.
     */
    private RasterLayer  layer;
    private MapToXMLUtils util;

    /**
     * Constructor with layer.
     *
     * @param layer The ArcGIS layer object.
     */
    public RasterLayerReader(ILayer layer)
    throws Exception {
        if(layer instanceof RasterLayer) {
            this.layer = (RasterLayer)layer;
        }
        else {
            throw new Exception("Not an instance of RasterLayer: " +
                layer.getClass().toString());
        }
    }

    /**
     * Setter for XML document helper.
     *
     * @param util The helper for storing map information.
     */
    public void setUtil(MapToXMLUtils util) {
        this.util = util;
    }

    /**
     * Reads the Layer content.
     *
     * @return The layer XML element.
     */
    public Element read()
    throws IOException{
        logger.debug("read()");
        Element layerElement;
        try {
            layerElement = util.addLayer();
        }
        catch(Exception e) {
            logger.error("Failed to create DOM-Element for Layer.");
            return null;
        }

        try {
            layerElement.setAttribute("name", layer.getName());
        }
        catch(Exception e) {
            logger.warn(
                "Could not read layer name." +
                " Stopped reading layer.");
            throw new IOException("Error reading layer name.");
        }

        try {
            layerElement.setAttribute("min_scale",
                String.valueOf(layer.getMinimumScale()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read minimum scale.");
        }

        try {
            layerElement.setAttribute("max_scale",
                String.valueOf(layer.getMaximumScale()));
        }
        catch(Exception e) {
            logger.warn(
                "Could not read maximum scale.");
        }

        try {
            if(layer.isVisible()) {
                layerElement.setAttribute("status", "on");
            }
            else {
                layerElement.setAttribute("status", "off");
            }
        }
        catch(Exception e) {
            logger.warn(
                "Could not read layer status." +
                " Setting layer status to \"on\".");
            layerElement.setAttribute("status", "on");
        }
        try {
            layerElement.setAttribute("data_source", layer.getFilePath());
        }
        catch(Exception e) {
            logger.warn(
                "Could not read datasource." +
                " Stopped reading layer " + layer.getName() + ".");
            util.removeLayer(layerElement);
            return null;
        }
        try {
            Envelope rect = (Envelope)layer.getExtent();
            layerElement.setAttribute(
                "extent_min_x",
                String.valueOf(rect.getXMin ()));
            layerElement.setAttribute(
                "extent_max_x",
                String.valueOf(rect.getXMax()));
            layerElement.setAttribute(
                "extent_min_y",
                String.valueOf(rect.getYMin()));
            layerElement.setAttribute(
                "extent_max_y",
                String.valueOf(rect.getYMax()));
        }
        catch(Exception e) {
            logger.warn(
                "Could not read extent from layer "
                + layer.getName() + ".");
        }
        //Read the projection.
        try {
            ISpatialReference sr = layer.getSpatialReference();
            int projection = 0;
            if(sr instanceof ProjectedCoordinateSystem) {
                ProjectedCoordinateSystem pcs = (ProjectedCoordinateSystem)sr;
                projection = pcs.getFactoryCode();
            }
            else if(sr instanceof GeographicCoordinateSystem) {
                GeographicCoordinateSystem gcs = (GeographicCoordinateSystem)sr;
                projection = gcs.getFactoryCode();
            }
            else if(sr instanceof UnknownCoordinateSystem) {
                UnknownCoordinateSystem ucs = (UnknownCoordinateSystem)sr;
                projection = 0;
            }
            else{
                logger.debug(
                    "Unknown SpatialReference: " +
                    sr.getClass().toString());
            }

            if(projection == 0) {
                logger.warn(
                    "Unknown projection for Layer:" + layer.getName() +
                    " Please edit projection in resulting mapfile.");
            }
            layerElement.setAttribute("projection", String.valueOf(projection));
        }
        catch(Exception e) {
            logger.warn("Could not read layer projection.");
        }

        // Static Attributes for Raster:
        layerElement.setAttribute("type", "raster");
        layerElement.setAttribute("connection_type", "local");

        return layerElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
